function [avsrc, classnum, mixmat] = SgenSS(chip,gene,red,srcnb,app,NL,stab,nbsimul,thresh,pval,fileid,sort)

% CgenSS -   COnsensus Source Separation for expression data
%          (based upon FastICA, by Hyvarinen et al)
%
%  usage:    [avsrc, src] =
%                CgenSS(chip,gene,red,srcnb,app,NL,stab,nbsimul,thresh,pval,fileid,sort)
%
%
% INPUT VARIABLES
% ---------------
% chip:        (float array): ngenes X ncond
%                   contains (logs of) expression data
% gene:        (char array): column vector of size ngenes
%                   contains gene names
% red:         (integer): positive, less than ncond
%                   value of reduced space dimension
% srcnb:        (integer): positive, less than red
%                   number of independent sources
% app:         (string): values 'defl' or 'symm'
%                   approach used for optimization:
%                   deflation or symmetrized
% NL:          (string): values 'pow3', 'tanh', 'gauss' or 'skew'
%                   nonlinearity
% stab:        (string) Values 'on' or 'off' (default value: 'off'). 
%                   controls wether the program uses the stabilized
%                   version of the algorithm or not.
% nbsimul:     (integer): positive
%                   number of ICA runs
% thresh:      (float): positive, less than 1
%                   threshold for the (normalized) scalar product
%                   of estimated sources
% pval:        (float): between 0 and 1
%                   p-value for selection of most significant genes
%                   in given independent components.
% fileid:      (string)
%                   name of output file.
% sort:        (string): values 'on' or 'off'
%                   if set to 'on', writes genes in alphabetical
%                   order to output file
%
%
% OUTPUT VARIABLES
% ----------------
% src:          (float array): srcnb X ngenes
%                   estimates for independent components
% classnum:    (cell array)
%                   class numbers
% mixmat:      (float array): ncond X srcnb
%                   mixing matrix
%%%%

% Backup
% ------
GenSSData{1} = red;
GenSSData{2} = srcnb;
GenSSData{3} = app;
GenSSData{4} = NL;
GenSSData{5} = stab;
GenSSData{6} = nbsimul;
GenSSData{7} = thresh;
GenSSData{8} = pval;
GenSSData{9} = fileid;
GenSSData{10} = sort;


Dim = size(chip);
ngenes = Dim(1);

% Do source separation
% --------------------
%figure(1);

h = waitbar(0,'Estimate sources');
for simul=1:nbsimul,
  waitbar(simul/nbsimul,h);
  src{simul} = fastica(chip',...
		       'lastEig',...
		       red,...
		       'numOfIC',...
		       srcnb,...
		       'g',...
		       NL,...
		       'approach',...
		       app,...
		       'stabilization',...
		       stab,...
		       'verbose',...
		       'off',...
		       'displayMode',...
		       'off'...
		       );
end;
close(h);

% for each simulation, the rows of src{simul} contain the sources
h = waitbar(0,'Normalize sources');
%allsrc = [];
for simul=1:nbsimul,
  tmp=src{simul};
  for k=1:srcnb,
  waitbar(simul/nbsimul,h);
    tmp(k,:) = tmp(k,:)/norm(tmp(k,:));
  end;
  src{simul} = tmp;
%  allsrc = [allsrc;src{simul}];
end;
close(h);

% Estimate "srcnb" average sources
[avsrc,counter] = GatherSrc(src, thresh, srcnb);

% Estimate average mixing matrix
mixmat = ConsSrc(chip,avsrc);
%figure; discr(mixmat,srcnb);
classnum = SrcAn(avsrc,gene,GenSSData);

%figure(2);
%for k=1:srcnb,
%  subplot(srcnb,1,k);,
%  hist(avsrc(k,:),30);,
%  title(['Source ' num2str(k)]);
%end;


% Report results
% ---------------
%GenSSReport(avsrc,gene, GenSSData, counter, fileid);
GenSSReport(avsrc,gene, classnum, GenSSData, counter,fileid);



  



  

