function ldata = takelog(data,method,factor);
%
%
% takelog -   Computation of logarithm of expression data
%
%  usage: ldata = takelog(data,method,factor);
%  
%
%
%  computation of logarithm after correction of zero values
%  the correction is performed by adding uniform noise,
%  distributed in [0,maxlevel]
%  
%  Four methods may be used for correcting zeros:
%
%    global:          constant maxlevel;
%                     noise added to all values.
%    masked global:   constant maxlevel;
%                     noise added to zero values only.
%    adaptive:        maxlevel is condition dependent
%                     noise added to all values.
%    masked adaptive: maxlevel is condition dependent;
%                     noise added to zero values only.
%  
%
%  Input variables:
%  ---------------
%  data: original data: ngenes X ncond
%  method: 'g','mg','a','ma' 
%  factor: normalization factor for added noise.
%
%  Output variables:
%  ----------------
%  ldata: corrected log of data
%


u = size(data);
ncond =u(2);
ngene = u(1);
mask = (data==0);,

if (method == 'g'),
  minval = min(min(data + 10*mask));,
subplot(2,1,1);
  tmp = reshape(data,ncond*ngene,1);
  plotdens(tmp,std(tmp)/5);
  title('Pdf of data: global correction');,
subplot(2,1,2);
  ldata = log(data + factor * minval * rand(u));,
  tmp = reshape(ldata,ncond*ngene,1);
%  hist(reshape(tmp,100);,
  plotdens(tmp,std(tmp)/5);
  title('Pdf of logarithms: global correction');,

elseif (method == 'mg'),
  minval = min(min(data + 10*mask));,
subplot(2,1,1);
  tmp = reshape(data,ncond*ngene,1);
  plotdens(tmp,std(tmp)/5);
  title('Pdf of data: masked global correction');,
subplot(2,1,2);
  ldata = log(data + factor * minval * rand(u).*mask);,
  tmp = reshape(ldata,ncond*ngene,1);
%  hist(reshape(tmp,100);,
  plotdens(tmp,std(tmp)/5);
  title('Pdf of logarithms: masked global correction');,

elseif (method=='a'),
subplot(2,1,1);
  tmp = reshape(data,ncond*ngene,1);
  plotdens(tmp,std(tmp)/5);
  title('Pdf of data: adaptive correction');,
subplot(2,1,2);
  for (cond=1:ncond),
    sdata = data(:,cond);,
    minval = min(sdata(find(sdata)));,
    ldata(:,cond) = log(sdata + factor * minval * rand(ngene,1));,
  end;
  tmp = reshape(ldata,ncond*ngene,1);
%  hist(reshape(tmp,100);,
  plotdens(tmp,std(tmp)/5);
  title('Pdf of logarithms: adaptive correction');,

elseif (method=='ma'),
subplot(2,1,1);
  tmp = reshape(data,ncond*ngene,1);
  plotdens(tmp,std(tmp)/5);
  title('Pdf of data: masked adaptive correction');,
subplot(2,1,2);
  for (cond=1:ncond),
    sdata = data(:,cond);,
    minval = min(sdata(find(sdata)));,
    noise = rand(ngene,1) .* (sdata==0);,
    ldata(:,cond) = log(sdata + factor * minval * noise);,
  end;
  tmp = reshape(ldata,ncond*ngene,1);
%  hist(reshape(tmp,100);,
  plotdens(tmp,std(tmp)/5);
  title('Pdf of logarithms: masked adaptive correction');,

end;




